/******************************************************************************
*	Module: dcf77.c
*	Decoding fr DCF77 radio clock (Germany Mainflingen 77,5kHz)
*
* 	Copyright (C) 2004 Volker Troyke, www.troyke.de
*
* 	This program is free software; you can redistribute it and/or
*	modify it under the terms of the GNU General Public License
*	as published by the Free Software Foundation; either version 2
*	of the License, or (at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program; if not, write to the Free Software
*	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

#include "global.h"
#include "dcf77.h"

#include <avr/interrupt.h>

// 30.12.2001	-	Bug fixed, 1 second intervall was too long -> TICK_1S - 1 ist correct
// 05.12.2003 	-	Evaluation of Summertime Flag added, Time-Structure changes to add additional SZ-Flag

u08 PROGMEM tage_im_monat[] = {0,31,28,31,30,31,30,31,31,30,31,30,31}; // ohne schaltjahr :-)

u16 dcf77_timebase, dcf77_timeout, dcf77_time, dcf77_tick;
u08 dcf77_update, inc_time;

u08 dcf_second, signal_ok, sync_ok, dcf_ok, dcf_valid, wait_falling, update_time;

u08 temp, parity;

#ifdef DCF77_DEBUG
u16 debug_low, debug_high;
#endif

void increment_time(struct dt *date)
{
	u08 temp;

	if(date->t.s != 59)
	{
		date->t.s++;
	}
	else
	{
		date->t.s=0;
		if(date->t.m != 59)
		{
			date->t.m++;
		}
		else
		{
			date->t.m=0;
			if(date->t.h != 23)
			{
				date->t.h++;
			}
			else
			{
				date->t.h=0;

				if(date->d.wt != 7)
				{
					date->d.wt++;
				}
				else
				{
					date->d.wt = 1;
				}

				if(((date->d.jahr & 0x03) == 0) && date->d.monat == 2) // Schaltjahr und Monat Februar
				{
					temp = 29;
				}
				else
				{
					temp = pgm_read_byte(&tage_im_monat[date->d.monat]);
				}

				if(date->d.tag != temp)
				{
					date->d.tag++;
				}
				else
				{
					date->d.tag=1;
					if(date->d.monat != 12)
					{
						date->d.monat++;
					}
					else
					{
						date->d.monat=1;
						date->d.jahr++;
					}
				}
			}
		}
	}
}

void dcf77_timetick(void)
{
	if(dcf77_tick == (TICK_1S - 1))	// fixed 30.12.2001
	{
#ifdef NTP_TIMESTAMP
		ntp_time_i++;
#endif
		dcf77_tick = 0;
		inc_time = 1;
	}
	else
	{
		dcf77_tick++;
	}

	dcf77_timebase++;

	if(dcf77_timebase >= DCF77_TIMEOUT)
	{
		signal_ok 	= 0;
		sync_ok 		= 0;
		dcf_ok		= 0;
		dcf_valid	= 0;
	}
}

SIGNAL(DCF77_INTERRUPT)   /* signal handler for interrupt 0 */
{
	dcf77_time = dcf77_timebase;
	dcf77_timebase = 0;
	if(wait_falling)
	{
		dcf77_update = 1;
		// fallende flanke detectiert
		wait_falling = 0;
#ifdef DCF77_DEBUG
		debug_low = dcf77_time;
#endif
#ifndef CPU_TYPE
		MCUCR=0x03; // switch to rising edge
#else
		EICRA = 0b00000011;			  // Int 0 rising Edge
#endif
	}
	else
	{
		wait_falling = 1;
#ifdef DCF77_DEBUG
		debug_high = dcf77_time;
#endif
#ifndef CPU_TYPE
		MCUCR=0x02; // switch to falling edge
#else
		EICRA = 0b00000010;			  // Int 0 falling Edge
#endif


		// steigende flanke detectiert

		if(dcf77_time > MIN_LOW && dcf77_time < MAX_LOW)
		{
			if(sync_ok)
			{
				dcf_second++;
				dcf_date.t.s = dcf_second;
			}

			signal_ok = 1;
		}
		else
		{
			if(dcf77_time > MIN_LOW * 2  && dcf77_time < MAX_LOW * 2 )
			{
				signal_ok 	= 1;

				if(dcf_second == 58)
				{
					// signal ok
					dcf_valid	= dcf_ok;
					if(dcf_valid)
					{
						dcf77_tick 		= 0;
						dcf_date.t.s 	= 0;
						update_time  	= 1;
					}
				}
				else
				{
					dcf_valid = 0;
				}

				sync_ok		= 1;
				dcf_second 	= 0;
			}
			else
			{
				dcf_second 	 = 0;
				signal_ok 	 = 0;
				sync_ok 		 = 0;
				dcf_ok		 = 0;
				dcf_valid	 = 0;
			}
		}
	}
}

void dcf77_process_data(void)
{
	u08 bit;

	bit = 0;

	if(!wait_falling)
	{
		if(dcf77_time > MIN_HIGH && dcf77_time < MAX_HIGH)
		{
			bit = 0;
		}
		else
		{
			if(dcf77_time > MIN_HIGH * 2 && dcf77_time < MAX_HIGH * 2)
			{
				bit = 1;
			}
			else
			{
				dcf_second	= 0;
				signal_ok 	= 0;
				sync_ok 		= 0;
				dcf_ok		= 0;
				dcf_valid	= 0;
			}
		}

		if(sync_ok)
		{
			// get time from dcf
			switch(dcf_second)
			{
				case  5 :
				{
					dcf_ok = 1;
				}
				break;

				case 20 :
				{
					if(!bit)
					{
						dcf_ok = 0;
					}
				}
				break;

            case 21: /* minute */
            case 29: /* hour */
            case 36: /* day */
            {
            	parity 				= bit;
            	temp   				= bit;
            }
            break;

            case 42: /* weekday */
            {
					dcf_date.d.tag 	= temp;
               parity 				^= bit;
               temp 					= bit;
            }
            break;

            case 45: /* month */
            {
               dcf_date.d.wt 		= temp;
               parity 				^= bit;
               temp 					= bit;
            }
            break;

            case 50: /* year */
            {
               dcf_date.d.monat 	= temp;
               parity 				^= bit;
               temp 					= bit;
            }
            break;

            case 22: /* minute */
            case 30: /* hour */
            case 37: /* day */
            case 43: /* weekday */
            case 46: /* month */
            case 51: /* year */
            {
               parity 				^= bit;
               temp 					+= (bit<<1);
            }
            break;

            case 23: /* minute */
            case 31: /* hour */
            case 38: /* day */
            case 44: /* weekday */
            case 47: /* month */
            case 52: /* year */
            {
               parity 				^= bit;
               temp 					+= (bit<<2);
            }
            break;

            case 24: /* minute */
            case 32: /* hour */
            case 39: /* day */
            case 48: /* month */
            case 53: /* year */
            {
               parity 				^= bit;
               temp 					+= (bit<<3);
            }
            break;

            case 25: /* minute */
            case 33: /* hour */
            case 40: /* day */
            case 49: /* month */
            case 54: /* year */
            {
               parity 				^= bit;
               if (bit) temp		+=	10;
            }
            break;

            case 26: /* minute */
            case 34: /* hour */
            case 41: /* day */
            case 55: /* year */
            {
               parity 				^= bit;
               if (bit) temp		+=	20;
            }
            break;

            case 27: /* minute */
            case 56: /* year */
            {
               parity 				^= bit;
               if (bit) temp		+=	40;
            }
            break;

            case 57: /* year */
            {
               parity 				^= bit;
               if (bit) temp		+=	80;
            }
            break;

            case 28: /* minute */
            {
               dcf_date.t.m = temp;

                if (bit != parity)
                {
                	dcf_ok = 0;
                }
            }
            break;

            case 35: /* hour */
            {
               dcf_date.t.h = temp;
               if (bit != parity)
               {
               	dcf_ok = 0;
               }
            }
            break;

            case 58: /* day, month, year */
            {
	          	dcf_date.d.jahr = temp;

               if (bit !=parity)
					{
						dcf_ok = 0;
					}
            }
            break;

            case 15:		// R-Bit, Normal 0, Reserveantenne 1
            	temp = (bit << 3);
            break;

            case 17:		// Z1-Bit, Winter 0, Sommer 1
            {
            	temp |= (bit << 2);
            }
            break;

            case 18: 	// Z2-Bit, Winter 1, Sommer 0
            {
            	temp |= (bit << 1);
            	if((temp & 0x06) == 0x02)
            	{
            		 dcf_date.t.timezone = temp; // Winterzeit
            	}
					else
					{
	            	if((temp & 0x06) == 0x04)
	            	{
	            		 dcf_date.t.timezone = (temp | 0x01); // Sommerzeit
	            	}
	            }
            }
            break;
			}
		}
	}
}

void dcf77_init(void)
{
	signal_ok 		= 0;
	sync_ok			= 0;
	dcf_ok			= 0;
	dcf_valid		= 0;
	update_time		= 0;
	inc_time 		= 0;

	dcf_date.d.jahr=0;
	dcf_date.d.monat=0;
	dcf_date.d.tag=0;
	dcf_date.d.wt=SA;
	dcf_date.t.h=0;
	dcf_date.t.m=0;
	dcf_date.t.s=0;
	dcf_date.t.timezone=0;
}
