/******************************************************************************
*	Module: dcf77.h
*	Decoding fr DCF77 radio clock (Germany Mainflingen 77,5kHz)
*
* 	Copyright (C) 2004 Volker Troyke, www.troyke.de
*
* 	This program is free software; you can redistribute it and/or
*	modify it under the terms of the GNU General Public License
*	as published by the Free Software Foundation; either version 2
*	of the License, or (at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program; if not, write to the Free Software
*	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

#ifndef DCF77_H
#define DCF77_H
#include "ds1302.h"

#define MAX_LOW 				 F_CPU /   TIMER_PRESCALE  					// 1s
#define MIN_LOW 				 F_CPU / ((TIMER_PRESCALE *   4L) / 3L )	// 0,75s  (3/4s)

#define MAX_HIGH 				 F_CPU / ((TIMER_PRESCALE *  85L) / 10L)	// 117ms (1/85s  /10)
#define MIN_HIGH 				 F_CPU / ((TIMER_PRESCALE * 120L) / 10L)	// 83 ms (1/120s /10)


#define DCF77_TIMEOUT 		 F_CPU / (TIMER_PRESCALE / 3)  				// 3s
#define TICK_1S				 F_CPU /  TIMER_PRESCALE
#define DCF77_INTERRUPT		SIG_INTERRUPT0

#define DCF77_DEBUG
#define NTP_TIMESTAMP

void dcf77_init(void);
void dcf77_timetick(void);
void increment_time(struct dt *date);
void dcf77_process_data(void);

extern struct dt act_date;
extern struct dt dcf_date;
extern u08 PROGMEM tage_im_monat[];
extern u08 dcf77_update;
extern u08 update_time;
extern u08 inc_time;
extern u08 signal_ok;
extern u08 dcf_valid;
extern u16 dcf77_tick;

#ifdef DCF77_DEBUG
extern u16 debug_low, debug_high;
extern u08 dcf_second;
#endif

#ifdef NTP_TIMESTAMP
extern u32 ntp_time_i;
#endif

#endif

