/******************************************************************************
*	Module: global.h
*	NTP-Server synchronized to DCF77 Radio Clock (Germany Mainflingen 77,5kHz)
*
*	Funtion ethernet_process_data based on:
*  The uIP TCP/IP stack code author Adam Dunkels <adam@dunkels.com>
*
* 	Copyright (C) 2004 Volker Troyke, www.troyke.de
*
* 	This program is free software; you can redistribute it and/or
*	modify it under the terms of the GNU General Public License
*	as published by the Free Software Foundation; either version 2
*	of the License, or (at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program; if not, write to the Free Software
*	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

#ifndef GLOBAL_H
#define GLOBAL_H

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>
#include <stdlib.h>

#include <../avr_make/vt_defs.h>


// ---- Global / SNAP Definitions -----------------------------------------------------------------------

#define TIMER_PRESCALE  8000										// Timer Prescale Factor = (ClockDivider * MaxCountOfTimer)
																			// Example: Divider  8, Timer Overflow 8-Bit (256) ->  8 * 256 = 2048
																			// Example: Divider 64, Timer reset on 124   (125) -> 64 * 125 = 8000

#define F_CPU        	16384000L               			// CPU-Clockspeed in Hz
#define TICK_100M			(F_CPU / (TIMER_PRESCALE * 10L))	// timer 0 prescaler for 100ms tick
#define TICK_UIP			(F_CPU / (TIMER_PRESCALE * 100L))	// timer 0 prescaler for ethernet periodic 10ms timetick

#define PORT_UART			10000

#define DISPLAY_IP_TIME 60											// time in seconds to display IP of NTP-Requests or RS232-Connects
// ----------------------------------------------------------------------------------------------------


// #define DISPLAY_MAC
#define NTP_UDP_LISTEN 1

#define UIP_APPSTATE_SIZE 1
#define UIP_APPCALL  tcp_appcall
#define FS_STATISTICS 0

#define RTS PB7
#define DDR_RTS 	DDRB
#define PORT_RTS	PORTB

#define IPRESET PA7
#define DDR_IPRESET PORTA
#define PORT_IPRESET PORTA
#define PIN_IPRESET	PINA

#define DATABASE_BASE  	0x00

struct _database
{
	u08 baud_rate;
	u16 uart_timeout;
};


#define MAXBAUD 8

struct PROGMEM _baud
{
	char text[5];
	u08 baud_reg_h;
	u08 baud_reg_l;
};


void tcp_appcall(void);
void udp_appcall(void);

#define IP_CONFIG_BASE	0x3E0

struct _ipconfig
{
	u16 hostadr[2];
	u16 netmask[2];
	u16 gateway[2];
};

extern u16 uart_connect_ip[2];
extern u16 tx_bytes, rx_bytes;
extern u16 tx_overflow, rx_overflow;
extern char data[6];
extern struct _ipconfig ipconfig;
extern struct _database database;
extern struct _baud baud[MAXBAUD];
extern u08 connect;
extern u16 ntp_req_counter;
extern u16 ntp_last_connect_ip[2];
extern u16 telnet_connect_ip[2];
extern struct dt act_date;
extern u16 uart_timeout;

#endif




