/******************************************************************************
*	Module: lcd.h
*	Device Driver for HD44780U based Alphanumeric-LCD-Displays
*
* 	Copyright (C) 2004 Volker Troyke, www.troyke.de
*
* 	This program is free software; you can redistribute it and/or
*	modify it under the terms of the GNU General Public License
*	as published by the Free Software Foundation; either version 2
*	of the License, or (at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program; if not, write to the Free Software
*	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

#ifndef LCD_H
#define LCD_H

// -------- LCD-Display Port Definitions ----------------------------------------------
// Data output port
#define PORT_LCD_DATA					PORTC
#define DDR_LCD_DATA						DDRC
#define PIN_LCD_DATA						PINC

// Control-Lines-Port
#define DDR_LCD_CONTROL					DDRA
#define PIN_LCD_CONTROL 				PINA
#define PORT_LCD_CONTROL 				PORTA

#define LCD_RS								PA0
#define LCD_RW								PA1
#define LCD_EN								PA5

#define PORT_LCD_BL						PORTD
#define DDR_LCD_BL						DDRD
#define LCD_BL								PD7
// -------------------------------------------------------------------------------------

// LCD Timing Adaption (slow down)
#define STARTUP_FACTOR					(F_CPU / 13000L)
#define LCD_TIMEOUT 						(F_CPU / 10000L)

#define LCD_CONTROL				   	1
#define LCD_DATA					   	0

// Function Set
#define LCD_BUS_8 						1			// Define for Data-Bus Width = 8-Bit
#define LCD_TWO_LINES					1			// Define for Two-Line Display
//#define LCD_FOUR_LINES					1			// Define in addition to LCD_TWO_LINE for 4x20 Display
//#define CONTROLLER_KS0073				1			// 4-Line Adressing for KS0073 Controller
#define COLDSTART							1			// Extended Startup if power supply conditions not met
#define LCD_SETCHAR						1			// Include setchar routine
//#define LCD_BACKLIGHT					1			// Include Backlight Control
//#define LCD_BACKLIGHT_PWM				1

#define LCD_OFF							0x08
#define LCD_ON_CURSOR_OFF				0x0C
#define LCD_ON_CURSOR_ON				0x0E
#define LCD_ON_CURSOR_BLINK			0x0F
#define LCD_ON_CURSOR_LEFT				0x10

// low level funktions ******************************

void	lcd_wait_busy(void);
void 	lcd_write_byte(u08 data, u08 control);
u08  	lcd_read_byte(u08 control);

// public functions *********************************

void char2str(u16 data, char *s, u08 digits);
void char2strh(u08 data, char *s);
void lcd_init(void);
void lcd_mode(u08 mode);
void lcd_clear(void);
void lcd_gotoxy(u08 x, u08 y);
void lcd_puts(char *data);
void lcd_puts_P(const char *data);
void lcd_setchar(u08 code, u08 row, u08 data);
void lcd_bl_intens(u08 intens);

// **************************************************

#endif

