/******************************************************************************
*	Module: telnet.c
*	Simple Telnet for changing IP-Adresses
*
* 	Copyright (C) 2005 Volker Troyke, www.troyke.de
*
* 	This program is free software; you can redistribute it and/or
*	modify it under the terms of the GNU General Public License
*	as published by the Free Software Foundation; either version 2
*	of the License, or (at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program; if not, write to the Free Software
*	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/


#include "global.h"
#include "uip.h"
#include "telnet.h"
#include "uip_arp.h"
#include "dcf77.h"
#include "lcd.h"

u16 tnpos;
u08 inputtype;
u08 tn_inbuff[INBUFF_SIZE];
u08 tn_inpos;


void ip_addstring_P(const char *data)
{
	u08 pos = 0;
	while(PRG_RDB(&data[pos]) != 0x00 && pos != 255 && tnpos < UIP_BUFSIZE-56)
	{
		uip_appdata[tnpos]=(PRG_RDB(&data[pos]));
		tnpos++;
		pos++;
	}
}

void ip_addstring(char *data)
{
	u08 pos = 0;
	while(data[pos] != 0x00 && pos != 255 && tnpos < UIP_BUFSIZE-56)
	{
		uip_appdata[tnpos]=data[pos];
		tnpos++;
		pos++;
	}
}

void ip_sendstring(void)
{
	if(tnpos>0)
	{
		uip_send(uip_appdata, tnpos);
	}
}

void display_ip_telnet(u16_t *ip)
{
	char2str(ip[0] & 0x00FF, data,3);
	ip_addstring(data);
	ip_addstring_P(PSTR("."));

	char2str(ip[0] >> 8, data,3);
	ip_addstring(data);
	ip_addstring_P(PSTR("."));

	char2str(ip[1] & 0x00FF, data,3);
	ip_addstring(data);
	ip_addstring_P(PSTR("."));

	char2str(ip[1] >> 8, data,3);
	ip_addstring(data);
}

void telnet(void)
{
	u08 substring[6];
	u08 i;

	switch(inputtype)
	{
		case INPUT_IDLE:
			switch(uip_appdata[0])
			{
				case 0x03:
				case 'x':
					uip_close();
				break;

				case 'h':
				case '?':
					tnpos=0;
					ip_addstring_P(PSTR("\r\nh - this Help Page\r\n"));
					ip_addstring_P(PSTR("u - Show UART-Server status\r\n"));
					ip_addstring_P(PSTR("U - set UART speed (Baud Rate)\r\n"));
					ip_addstring_P(PSTR("T - set UART Close Connection Timeout\r\n"));
					ip_addstring_P(PSTR("d - Show NTP-Server status\r\n"));
					ip_addstring_P(PSTR("I - set IP\r\n"));
					ip_addstring_P(PSTR("M - set Subnetmask\r\n"));
					ip_addstring_P(PSTR("G - set Default-Gateway\r\n"));
					ip_addstring_P(PSTR("i - display IP-Configuration\r\n"));
					ip_addstring_P(PSTR("s - use new IP-Configuration\r\n"));
					ip_addstring_P(PSTR("S - save IP-Configuration permanent\r\n"));
					ip_addstring_P(PSTR("x - exit Telnet Session\r\n"));
					ip_sendstring();
				break;

				case 'u':
					tnpos = 0;

					ip_addstring_P(PSTR("\r\nUART connected to IP : "));
					if(connect)
					{
						display_ip_telnet(uart_connect_ip);
					}
					else
					{
						ip_addstring_P(PSTR("none, not connected"));
					}
					ip_addstring_P(PSTR("\r\nBytes   to RS232     : "));
					char2str(tx_bytes, data, 5);
					ip_addstring(data);
					ip_addstring_P(PSTR("\r\nBytes from RS232     : "));
					char2str(rx_bytes, data, 5);
					ip_addstring(data);
					ip_addstring_P(PSTR("\r\nTX-Buffer Overflows  : "));
					char2str(tx_overflow, data, 5);
					ip_addstring(data);
					ip_addstring_P(PSTR("\r\nRX-Buffer Overflows  : "));
					char2str(rx_overflow, data, 5);
					ip_addstring(data);
					ip_addstring_P(PSTR("\r\nUART Time to Close   : "));
					char2str(uart_timeout, data, 5);
					ip_addstring(data);
					ip_addstring_P(PSTR("\r\nUART Close Timeout   : "));
					char2str(database.uart_timeout, data, 5);
					ip_addstring(data);
					ip_addstring_P(PSTR("\r\nBaud Rate            : "));
					ip_addstring_P(baud[database.baud_rate].text);
					ip_addstring_P(PSTR("\r\n"));
					ip_sendstring();
					break;

				case 'U':
					tnpos=0;
					ip_addstring_P(PSTR("\r\nChoose Baud Rate:"));
					for(i=0;i<MAXBAUD;i++)
					{
						ip_addstring_P(PSTR("\r\n"));
						char2str(i,data,1);
						ip_addstring(data);
						ip_addstring_P(PSTR(" for "));
						ip_addstring_P(baud[i].text);
						ip_addstring_P(PSTR(" Baud"));
					}
					ip_addstring_P(PSTR("\r\n"));
					ip_sendstring();
					inputtype=INPUT_BAUD;
					tn_inpos=0;
				break;

				case 'd':
					tnpos=0;
					ip_addstring_P(PSTR("\r\nLocal Time               : "));
					switch(act_date.d.wt)
					{
						case MO: ip_addstring_P(PSTR("Mo")); break;
						case DI: ip_addstring_P(PSTR("Di")); break;
						case MI: ip_addstring_P(PSTR("Mi")); break;
						case DO: ip_addstring_P(PSTR("Do")); break;
						case FR: ip_addstring_P(PSTR("Fr")); break;
						case SA: ip_addstring_P(PSTR("Sa")); break;
						case SO: ip_addstring_P(PSTR("So")); break;
					}
					ip_addstring_P(PSTR(" "));
					if(act_date.d.tag < 10)
					{
						//ip_addstring_P(PSTR(" "));
						char2str(act_date.d.tag, data,1);
						ip_addstring(data);
					}
					else
					{
						char2str(act_date.d.tag, data,2);
						ip_addstring(data);
					}
					ip_addstring_P(PSTR("."));
					char2str(act_date.d.monat, data,2);
					ip_addstring(data);
					ip_addstring_P(PSTR(".20"));
					char2str(act_date.d.jahr, data,2);
					ip_addstring(data);
					ip_addstring_P(PSTR(" "));
					if(act_date.t.h < 10)
					{
						//ip_addstring_P(PSTR(" "));
						char2str(act_date.t.h, data,1);
						ip_addstring(data);
					}
					else
					{
						char2str(act_date.t.h, data,2);
						ip_addstring(data);
					}
					ip_addstring_P(PSTR(":"));
					char2str(act_date.t.m, data,2);
					ip_addstring(data);
					ip_addstring_P(PSTR(":"));
					char2str(act_date.t.s, data,2);
					ip_addstring(data);

					if(act_date.t.timezone & 0x01)
					{
						ip_addstring_P(PSTR(" MESZ"));
					}
					else
					{
						ip_addstring_P(PSTR(" MEZ"));
					}

					ip_addstring_P(PSTR("\r\nLast NTP-Request from IP : "));
					display_ip_telnet(ntp_last_connect_ip);

					ip_addstring_P(PSTR("\r\nNTP-Requests             : "));
					char2str(ntp_req_counter, data,5);
					ip_addstring(data);

					if(signal_ok)
					{
						ip_addstring_P(PSTR("\r\nDCF-Signal Quality       : OK"));
					}
					else
					{
						ip_addstring_P(PSTR("\r\nDCF-Signal Quality       : Fail!"));
					}

					if(dcf_valid)
					{
						ip_addstring_P(PSTR("\r\nDCF in Sync              : OK"));
					}
					else
					{
						ip_addstring_P(PSTR("\r\nDCF in Sync              : Fail!"));
					}

					ip_addstring_P(PSTR("\r\nSignal detection         : High="));
					char2str(debug_high, data,5);
					ip_addstring(data);
					ip_addstring_P(PSTR(" Low="));
					char2str(debug_low, data,5);
					ip_addstring(data);

					ip_addstring_P(PSTR("\r\nLogin von IP             : "));
					display_ip_telnet(telnet_connect_ip);

					ip_addstring_P(PSTR("\r\n"));

					ip_sendstring();
				break;

				case 'i':
					tnpos=0;
					ip_addstring_P(PSTR("\r\nIP-Adress    : "));
					display_ip_telnet(ipconfig.hostadr);
					ip_addstring_P(PSTR("\r\nSubnet-Mask  : "));
					display_ip_telnet(ipconfig.netmask);
					ip_addstring_P(PSTR("\r\nDefault-Gate : "));
					display_ip_telnet(ipconfig.gateway);
					ip_addstring_P(PSTR("\r\nUART PORT    : "));
					char2str(PORT_UART, data,5);
					ip_addstring(data);
					ip_addstring_P(PSTR("\r\n"));
					ip_sendstring();
				break;

				case 'I':
					tnpos = 0;
					ip_addstring_P(PSTR("\r\nEnter IP-Adress: xxx.xxx.xxx.xxx\r\n"));
					ip_sendstring();
					inputtype = INPUT_IP;
					tn_inpos = 0;
					break;

				case 'M':
					tnpos = 0;
					ip_addstring_P(PSTR("\r\nEnter Subnetz-Mask: xxx.xxx.xxx.xxx\r\n"));
					ip_sendstring();
					inputtype = INPUT_MASK;
					tn_inpos = 0;
					break;

				case 'G':
					tnpos = 0;
					ip_addstring_P(PSTR("\r\nEnter Default-Gateway: xxx.xxx.xxx.xxx\r\n"));
					ip_sendstring();
					inputtype = INPUT_GATE;
					tn_inpos = 0;
					break;

				case 'T':
					tnpos = 0;
					ip_addstring_P(PSTR("\r\nEnter UART-Connetion Timeout in 0.1s: xxxxx (65535=OFF)\r\n"));
					ip_sendstring();
					inputtype = INPUT_U_TIMEOUT;
					tn_inpos = 0;
					break;


				case 's':
					tnpos=0;
					uip_sethostaddr(ipconfig.hostadr);
					uip_setnetmask(ipconfig.netmask);
					uip_setdraddr(ipconfig.gateway);
					ip_addstring_P(PSTR("\r\nnow using new IP-Configuration\r\n"));
					ip_sendstring();
				break;

				case 'S':
					tnpos=0;
					eeprom_write_block(&ipconfig, (int*)IP_CONFIG_BASE, sizeof(struct _ipconfig));
					ip_addstring_P(PSTR("\r\nIP-Configuration Saved to EEPROM\r\n"));
					ip_sendstring();
				break;

				default:
				 // uip_send(uip_appdata, 2); // echo characters
				break;
			}
			break;

			case INPUT_IP:
			case INPUT_GATE:
			case INPUT_MASK:
			case INPUT_BAUD:
			case INPUT_U_TIMEOUT:
			switch(uip_appdata[0])
			{
				case 0x03:	// Ctrl-C ?
				case 0x1B: 	// ESC
				case 'x':
					tnpos=0;
					ip_addstring_P(PSTR("\r\n...abbruch\r\n"));
					ip_sendstring();
					inputtype=INPUT_IDLE;
				break;

				case 0x0D:	// Enter
					if(tn_inpos)
					{
						tn_inbuff[tn_inpos]=0; // terminate String with 0
						i = 0;
						u08 sub=0;
						u08 part=0;
						u16 adr_0=0, adr_1=0;

						switch(inputtype)
						{
							case INPUT_BAUD:

								tnpos=0;
								inputtype=INPUT_IDLE;
								i = atoi((char*)tn_inbuff);

								if(i<MAXBAUD)
								{
									database.baud_rate = i;
									eeprom_write_block(&database, (int*)DATABASE_BASE, sizeof(struct _database));

									ip_addstring_P(PSTR("\r\nBaud Rate is now: "));
									ip_addstring_P(baud[database.baud_rate].text);
									ip_addstring_P(PSTR(" Baud"));
									ip_addstring_P(PSTR("\r\n"));

									UBRRH=pgm_read_byte(&baud[database.baud_rate].baud_reg_h);		// set baud rate H-Register
									UBRRL=pgm_read_byte(&baud[database.baud_rate].baud_reg_l);		// set baud rate L-Register

									lcd_gotoxy(10,1);
									lcd_puts_P(baud[database.baud_rate].text);
								}
								else
								{
									ip_addstring_P(PSTR("\r\nBaud Rate not supported!\r\n"));
								}

								ip_sendstring();
							break;

							case INPUT_U_TIMEOUT:

								tnpos = 0;

								database.uart_timeout = atoi((char*)tn_inbuff);
								uart_timeout=database.uart_timeout;
								ip_addstring_P(PSTR("\r\nUART Close Connection Timeout is now: "));
								char2str(database.uart_timeout, data, 5);
								ip_addstring(data);
								ip_addstring_P(PSTR(" 1/10s"));

								eeprom_write_block(&database, (int *) DATABASE_BASE, sizeof(struct _database));

								ip_addstring_P(PSTR("\r\n"));
								inputtype = INPUT_IDLE;
								ip_sendstring();
								break;

							case INPUT_IP:
							case INPUT_GATE:
							case INPUT_MASK:
								while(i<tn_inpos && sub<6)
								{
									if(tn_inbuff[i]=='.')
									{
										substring[sub]=0;
										ip_addstring_P(PSTR("\r\n"));
										ip_addstring((char*)substring);
										switch(part)
										{
											case 0:
												adr_0 = atoi((char*)substring);
											break;
											case 1:
												adr_0 += atoi((char*)substring) <<8;
											break;
											case 2:
												adr_1 = atoi((char*)substring);
											break;
											case 3:
												adr_1 += atoi((char*)substring) <<8;
											break;
										}
										part++;
										sub=0;
									}
									else
									{
										substring[sub]=tn_inbuff[i];
										sub++;
									}
									i++;
								}
								substring[sub]=0;
								adr_1 += atoi((char*)substring) <<8;
								if(part >= 3)
								{
									switch(inputtype)
									{
										case INPUT_IP:
											ipconfig.hostadr[0]=adr_0;
											ipconfig.hostadr[1]=adr_1;
										break;
										case INPUT_MASK:
											ipconfig.netmask[0]=adr_0;
											ipconfig.netmask[1]=adr_1;
										break;
										case INPUT_GATE:
											ipconfig.gateway[0]=adr_0;
											ipconfig.gateway[1]=adr_1;
										break;
									}
								}
								inputtype=INPUT_IDLE;
								tnpos=0;
								ip_addstring_P(PSTR("\r\nIP-Adresse    : "));
								display_ip_telnet(ipconfig.hostadr);
								ip_addstring_P(PSTR("\r\nSubnetz-Maske : "));
								display_ip_telnet(ipconfig.netmask);
								ip_addstring_P(PSTR("\r\nDefault-Gate  : "));
								display_ip_telnet(ipconfig.gateway);
								ip_addstring_P(PSTR("\r\n"));
								ip_sendstring();
							break;
						}
					}
				break;

				case 0x08: // Backspace
				case 0x7F:
					tnpos=0;
					ip_addstring_P(PSTR("\b \b"));
					ip_sendstring();
					tn_inpos--;
				break;

				default:
					i=0;
				   while(tn_inpos < INBUFF_SIZE && i<uip_len)
					{
						tn_inbuff[tn_inpos]=uip_appdata[i];
						tn_inpos++;
						i++;
					}
					uip_send(uip_appdata, i);
				break;
			}
			break;
	}
}
