/******************************************************************************
*	Module: lcd.c
*	Device Driver for HD44780U based Alphanumeric-LCD-Displays
*
* 	Copyright (C) 2004 Volker Troyke, www.troyke.de
*
* 	This program is free software; you can redistribute it and/or
*	modify it under the terms of the GNU General Public License
*	as published by the Free Software Foundation; either version 2
*	of the License, or (at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program; if not, write to the Free Software
*	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/


#include "global.h"
#include "lcd.h"

// 26.08.2004 changed DDR Dataport switching, dataport stays to input after LCD access to make it compatible with RTL8019 Ethernet device driver
// 03.06.2005 LCD Backlight Function added, Intensity Control by PWM with Counter2 (AVRMega16/32), 4-Line LCD Support added (4x20 Display)

void char2str(u16 data, char *s, u08 digits)
{
	u16 div;
	u08 i, c;
	char temp[5];

	div = 10000;

	for(i=0;i<5;i++)
	{
		c = 0;
		while(data >= div)
		{
			data -= div;
			c++;
		};

		temp[i] = c + '0';
		div /= 10;
 	}

 	for(i=0;i<digits;i++)
 	{
		s[i]=temp[i- digits + 5];
 	}
	s[i] = 0;
}

void char2strh(u08 data, char *s)
{
	s[0] = (data >> 4) + '0';
	if(s[0] > '9') s[0]+= 7;
	s[1] = (data & 0x0F) + '0';
	if(s[1] > '9') s[1]+= 7;
	s[2] = 0x00;
}


//void char2str(u16 data, char *s, u08 digits)
//{
//	u16 div;
//	u08 i, c, n;
//	char temp[5];
//
//	div = 10000;
//
//	for(i=0;i<5;i++)
//	{
//		c = data / div;
//		temp[i] = c + '0';
//		data -= c * div;
//		div /= 10;
// 	}
//
// 	for(i=0;i<digits;i++)
// 	{
//		s[i]=temp[i- digits + 5];
// 	}
//	s[i] = 0;
//}

void short_delay(void)
{
}

void long_delay(volatile u16 delay)
{
	while(delay)
	{
		short_delay();
		delay--;
	};
}

void 	lcd_write_byte(u08 data, u08 control)
{
#ifndef LCD_BUS_8
	u08 temp;
#endif
	if(control == LCD_CONTROL)
	{
		cbi(PORT_LCD_CONTROL, LCD_RS);
	}
	else
	{
		sbi(PORT_LCD_CONTROL, LCD_RS);
	}
	cbi(PORT_LCD_CONTROL, LCD_RW); 	// Write Mode
	sbi(PORT_LCD_CONTROL, LCD_EN); 	// Enable ON

#ifdef LCD_BUS_8
	DDR_LCD_DATA 	= 0xFF;        	// DDR Data-Bus OUT
	PORT_LCD_DATA 	= data;
#else
	DDR_LCD_DATA 	|= 0xF0;       	// DDR Data-Bus OUT (upper 4-Bit only)
	temp = PORT_LCD_DATA;
	PORT_LCD_DATA = ((temp & 0x0F) | (data & 0xF0));

	cbi(PORT_LCD_CONTROL, LCD_EN); 	// Enable OFF
	sbi(PORT_LCD_CONTROL, LCD_EN); 	// Enable ON

	temp = PORT_LCD_DATA;
	PORT_LCD_DATA = ((temp & 0x0F) | (data << 4));
#endif

	cbi(PORT_LCD_CONTROL, LCD_EN); 	// Enable OFF

#ifdef LCD_BUS_8
	DDR_LCD_DATA	= 0x00;       		// DDR Data-Bus IN
	PORT_LCD_DATA	= 0xFF;				// Data Port enable Pullups
#else
	DDR_LCD_DATA 	&= 0x0F;      		// DDR Data-Bus IN (upper 4-Bit only)
	PORT_LCD_DATA	|= 0xF0;				// Data Port enable Pullups (upper 4-Bit only)
#endif
}

u08 lcd_read_byte(u08 control)
{
	u08 temp;
#ifndef LCD_BUS_8
	u08 temp2;
#endif

#ifdef LCD_BUS_8
	DDR_LCD_DATA	= 0x00;       		// DDR Data-Bus IN
	PORT_LCD_DATA	= 0xFF;				// Data Port enable Pullups
#else
	DDR_LCD_DATA 	&= 0x0F;      		// DDR Data-Bus IN (upper 4-Bit only)
	PORT_LCD_DATA	|= 0xF0;				// Data Port enable Pullups (upper 4-Bit only)
#endif

	if(control == LCD_CONTROL)
	{
	 	cbi(PORT_LCD_CONTROL, LCD_RS);
	}
	else
	{
		sbi(PORT_LCD_CONTROL, LCD_RS);
	}
	sbi(PORT_LCD_CONTROL, LCD_RW); // Read Mode
	sbi(PORT_LCD_CONTROL, LCD_EN); // Enable ON

#ifdef LCD_BUS_8
	long_delay(2);
	temp = PIN_LCD_DATA;
#else
	long_delay(2);
	temp = PIN_LCD_DATA & 0xF0;
	cbi(PORT_LCD_CONTROL, LCD_EN); // Enable OFF
	long_delay(2);
	sbi(PORT_LCD_CONTROL, LCD_EN); // Enable ON
	long_delay(2);
	temp2 = PIN_LCD_DATA;
	temp = temp | ((temp2 >> 4) & 0x0F);
#endif

	cbi(PORT_LCD_CONTROL, LCD_EN); // Enable OFF
	cbi(PORT_LCD_CONTROL, LCD_RW); // Write Mode

	return temp;
}

void lcd_wait_busy()
{
	u08 data;
	u16 timeout;

	timeout = LCD_TIMEOUT;
	do
	{
		data = lcd_read_byte(LCD_CONTROL);
		timeout--;
	}
	while((data & 0x80) && timeout);

#ifdef LCD_8_8
	if((data & 0x7F) == 0x07)
	{
		lcd_write_byte(0xC0, LCD_CONTROL);
		timeout = LCD_TIMEOUT;
		do
		{
			data = lcd_read_byte(LCD_CONTROL);
			timeout--;
		}
		while((data & 0x80) && timeout);
	}
#endif

}

void lcd_init()
{
#ifdef LCD_BACKLIGHT
	sbi(DDR_LCD_BL,  LCD_BL);	 		// Backlight Pin to OUT
#ifdef LCD_BACKLIGHT_PWM
#ifndef CPU_TYPE
	TCCR2	=	0x79;
	OCR2	=	0xff;
#else
// ATmega644
   TCCR2A = 0b11000011;
   TCCR2B = 0b00000001;
   OCR2A	 =	0xff;
#endif
#else
	cbi(PORT_LCD_BL, LCD_BL); 			// BL on
#endif
#endif

#ifdef COLDSTART
	u08 startup;
#endif
#ifdef LCD_BUS_8
	DDR_LCD_DATA 	= 0xFF;        	// DDR Data-Bus OUT
#else
	DDR_LCD_DATA 	|= 0xF0;
#endif

	sbi(DDR_LCD_CONTROL, LCD_RS);	 	// Control Lines to OUT
	sbi(DDR_LCD_CONTROL, LCD_RW);		// Control Lines to OUT
	sbi(DDR_LCD_CONTROL, LCD_EN);		// Control Lines to OUT

	cbi(PORT_LCD_CONTROL, LCD_EN); 	// Enable OFF
	cbi(PORT_LCD_CONTROL, LCD_RW); 	// Write Mode
	cbi(PORT_LCD_CONTROL, LCD_RS);	// Control Select

#ifdef COLDSTART
	for(startup=3;startup > 0;startup--)
	{
		long_delay(STARTUP_FACTOR * (1 << (startup * 2)));

		cbi(PORT_LCD_CONTROL, LCD_RS);	// Control Select
		sbi(PORT_LCD_CONTROL, LCD_EN); 	// Enable ON
#ifdef LCD_BUS_8
		PORT_LCD_DATA=0x3A;
#else
		PORT_LCD_DATA &=0x0F;
		PORT_LCD_DATA |=0x30;
#endif
		long_delay(STARTUP_FACTOR);
		cbi(PORT_LCD_CONTROL, LCD_EN); 	// Enable OFF
		long_delay(STARTUP_FACTOR);
	}
#endif

#ifdef LCD_BUS_8
	cbi(PORT_LCD_CONTROL, LCD_RS);	// Control Select
	sbi(PORT_LCD_CONTROL, LCD_EN); 	// Enable ON
	PORT_LCD_DATA=0x3A;
	long_delay(STARTUP_FACTOR);
	cbi(PORT_LCD_CONTROL, LCD_EN); 	// Enable OFF
#else
	cbi(PORT_LCD_CONTROL, LCD_RS);	// Control Select
	sbi(PORT_LCD_CONTROL, LCD_EN); 	// Enable ON
	PORT_LCD_DATA &= 0x0F;
	PORT_LCD_DATA |= 0x20;
	long_delay(STARTUP_FACTOR);
	cbi(PORT_LCD_CONTROL, LCD_EN); 	// Enable OFF
#endif
	long_delay(STARTUP_FACTOR);

#ifdef LCD_FOUR_LINES
	#ifdef LCD_BUS_8
		lcd_write_byte(0x3E, LCD_CONTROL);
		lcd_wait_busy();
		lcd_write_byte(0x09, LCD_CONTROL); // 4-Line KS0073 Controller
		lcd_wait_busy();
		lcd_write_byte(0x3A, LCD_CONTROL);
		lcd_wait_busy();
		lcd_write_byte(0x01 ,LCD_CONTROL);
	#else
		lcd_write_byte(0x2E, LCD_CONTROL);
		lcd_wait_busy();
		lcd_write_byte(0x09, LCD_CONTROL); // 4-Line KS0073 Controller
		lcd_wait_busy();
		lcd_write_byte(0x2A, LCD_CONTROL);
		lcd_wait_busy();
		lcd_write_byte(0x01 ,LCD_CONTROL);
	#endif
#else
	#ifdef LCD_TWO_LINES
		#ifdef LCD_BUS_8
			lcd_write_byte(0x3A, LCD_CONTROL);
		#else
			lcd_write_byte(0x2A, LCD_CONTROL);
		#endif
	#else
		#ifdef LCD_BUS_8
			lcd_write_byte(0x32, LCD_CONTROL);
		#else
			lcd_write_byte(0x22, LCD_CONTROL);
		#endif
	#endif
#endif

	lcd_wait_busy();
	lcd_write_byte(0x06, LCD_CONTROL);
}

void lcd_clear()
{
	lcd_wait_busy();
	lcd_write_byte(0x01 ,LCD_CONTROL);
}

void lcd_mode(u08 mode)
{
	lcd_wait_busy();
	lcd_write_byte(mode, LCD_CONTROL);
}

#ifdef LCD_SETCHAR
void lcd_setchar(u08 code, u08 row, u08 data)
{
	lcd_wait_busy();
	lcd_write_byte((code << 3) | row | 0x40, LCD_CONTROL);

	lcd_wait_busy();
	lcd_write_byte(data & 0x1F, LCD_DATA);
}
#endif

void lcd_puts(char *data)
{
	u08 pos;

	pos = 0;
	while(data[pos] != 0x00)
	{
		lcd_wait_busy();
		lcd_write_byte(data[pos], LCD_DATA);
		pos++;
	}
}

void lcd_puts_P(const char *data)
{
	u08 pos;

	pos = 0;
	while(pgm_read_byte(&data[pos]) != 0x00)
	{
		lcd_wait_busy();
		lcd_write_byte(pgm_read_byte(&data[pos]), LCD_DATA);
		pos++;
	}
}

#ifdef LCD_BACKLIGHT
void lcd_bl_intens(u08 intens)
{
#ifdef LCD_BACKLIGHT_PWM
#ifndef CPU_TYPE
	OCR2 = intens;
#else
// ATmega644
	OCR2A = intens;
#endif
#else
	if(intens)
	{
		cbi(PORT_LCD_BL, LCD_BL); 			// BL on
	}
	else
	{
		sbi(PORT_LCD_BL, LCD_BL); 			// BL off
	}
#endif
}
#endif

#ifdef LCD_FOUR_LINES
void lcd_gotoxy(u08 x, u08 y)
{
	u08 adr;

#ifndef CONTROLLER_KS0073
	switch(y)
	{
		case 1:
			adr=0x00;
		break;
		case 2:
			adr=0x40;
		break;
		case 3:
			adr=0x14;
		break;
		case 4:
			adr=0x54;
		break;
		default:
			adr=0x00;
	}
#else
	switch(y)
	{
		case 1:
			adr=0x00;
		break;
		case 2:
			adr=0x20;
		break;
		case 3:
			adr=0x40;
		break;
		case 4:
			adr=0x60;
		break;
		default:
			adr=0x00;
	}
#endif

	adr |= 0x80;
	adr |= (x-1);
	lcd_wait_busy();
	lcd_write_byte(adr, LCD_CONTROL);
}
#else
void lcd_gotoxy(u08 x, u08 y)
{
	u08 adr;
#ifndef LCD_8_8
	adr = 0;
	adr = (0x80 | (x-1) | ((y-1)<<6));
	lcd_wait_busy();
	lcd_write_byte(adr, LCD_CONTROL);
#else
	adr = 0;
	adr = (0x80 | ((x-1) & 0x07) | (((x-1) & 0xF8) <<3));
	lcd_wait_busy();
	lcd_write_byte(adr, LCD_CONTROL);
#endif
}
#endif
